@@ -1,4 +1,4 @@
-use anyhow::Result;
+use anyhow::{bail, Result};
 use base16::encode_lower;
 use macro_rules_attribute::apply;
 use pyo3::{
@@ -9,6 +9,7 @@ use rr_util::{
     lru_cache::TensorCacheRrfs,
     py_types::{scalar_to_tensor, Tensor, PY_UTILS},
     sv,
+    symbolic_size::{shape_first_symbolic_dim, SymbolicSizeProduct},
     tensor_db::{get_tensor_prefix, save_tensor},
     tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp},
 };
@@ -18,7 +19,7 @@ use uuid::Uuid;
 use crate::{
     circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeComputeInfoImpl, CircuitNodeHashItems},
-    new_rc,
+    new_rc, new_rc_unwrap,
     prelude::*,
     CachedCircuitInfo, CircuitFlags, PyCircuitBase, Shape, TensorEvalError,
 };
@@ -85,16 +86,21 @@ impl CircuitNode for Array {
 }
 
 impl Array {
-    #[apply(new_rc)]
-    pub fn new(value: Tensor, name: Option<String>) -> (Self) {
+    #[apply(new_rc_unwrap)]
+    pub fn try_new(value: Tensor, name: Option<String>) -> Result<Self> {
         let value = value.hashed();
+        if let Some(i) = shape_first_symbolic_dim(value.shape()) {
+            bail!(ConstructError::ArrayHasReservedSymbolicShape {
+                number: value.shape()[i],
+                sym: SymbolicSizeProduct::from(value.shape()[i]).to_string(),
+            })
+        }
         Self {
             value,
             name,
             info: Default::default(),
         }
         .initial_init_info()
-        .unwrap()
     }
     pub fn from_hash_prefix(
         name: Option<String>,
@@ -126,8 +132,8 @@ impl Array {
 impl Array {
     #[new]
     #[args(name = "None")]
-    fn py_new(value: Tensor, name: Option<String>) -> PyClassInitializer<Self> {
-        Array::new(value, name).into_init()
+    fn py_new(value: Tensor, name: Option<String>) -> Result<PyClassInitializer<Self>> {
+        Ok(Array::try_new(value, name)?.into_init())
     }
 
     #[staticmethod]
