@@ -7,8 +7,8 @@ use anyhow::{anyhow, bail, Context, Result};
 use itertools::Itertools;
 use pyo3::prelude::*;
 use rand::Rng;
-use rr_util::{fn_struct, name::Name, py_types::MaybeNotSet, simple_default};
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use rr_util::{fn_struct, py_types::MaybeNotSet, simple_default};
+use rustc_hash::FxHashSet as HashSet;
 
 use crate::{
     opaque_iterative_matcher::{get_opaque_type_matcher, OpaqueIterativeMatcherVal},
@@ -95,26 +95,11 @@ impl PrintHtmlOptions {
             selfy: &PrintHtmlOptions,
             overall_traversal: Option<OpaqueIterativeMatcherVal>,
             node_comments: Vec<String>,
-            parent_info: Option<(&Circuit, usize)>,
         ) -> Result<()> {
-            let printer = PrintOptions::default();
             if !seen.insert(circ.info().hash) {
                 return Ok(());
             }
             let mut desc: String = selfy.repr_line(circ.clone())?;
-            parent_info.map(|(parent, child_idx)| {
-                // get_child_info is very tied to regular printer, hand rolling
-                parent.as_module().map(|m| {
-                    child_idx.checked_sub(1).map(|i| {
-                        let arg_spec = &m.spec.arg_specs[i];
-                        arg_spec
-                            .symbol
-                            .info()
-                            .name
-                            .map(|n| write!(desc, " ! '{}'", n));
-                    })
-                });
-            });
             write_comment(&mut desc, node_comments, &selfy.commenters, circ.crc())?;
             make_str_safe(&mut desc)?;
             let mut color = match selfy
@@ -126,7 +111,7 @@ impl PrintHtmlOptions {
                 None => selfy.primary_color.clone(),
             };
             make_str_css_safe(&mut color)?;
-            let mut name = Name::string_maybe_empty(circ.info().name);
+            let mut name = circ.name_cloned().unwrap_or("".to_owned());
             make_str_safe(&mut name)?;
             write!(
                 nodes_string,
@@ -137,8 +122,19 @@ impl PrintHtmlOptions {
                 &color,
             )
             .unwrap();
-            for c in circ.children() {
-                write!(nodes_string, "\"{}\",", c.hash_base16()).unwrap();
+
+            for (child_idx, c) in circ.children().enumerate() {
+                write!(nodes_string, "[\"{}\"", c.hash_base16()).unwrap();
+                circ.as_module().map(|m| {
+                    child_idx.checked_sub(1).map(|i| {
+                        let arg_spec = &m.spec.arg_specs[i];
+                        arg_spec
+                            .symbol
+                            .name()
+                            .map(|n| write!(nodes_string, " ! '{}'", n));
+                    })
+                });
+                write!(nodes_string, "],").unwrap();
             }
             nodes_string.push_str("]},");
 
@@ -151,7 +147,6 @@ impl PrintHtmlOptions {
                 // all finished case
                 return Ok(());
             };
-            let give_parent_info = printer.has_child_info(&circ);
             for (i, (child, new_traversal)) in circ.children().zip(new_traversal_per).enumerate() {
                 let child_specific_commenters = if selfy.comment_arg_names {
                     get_child_comments(&(**circ), i)
@@ -165,11 +160,6 @@ impl PrintHtmlOptions {
                     selfy,
                     new_traversal,
                     child_specific_commenters,
-                    if give_parent_info {
-                        Some((&circ, i))
-                    } else {
-                        None
-                    },
                 )?;
             }
             Ok(())
@@ -182,7 +172,6 @@ impl PrintHtmlOptions {
                 &self,
                 self.overall_traversal.clone(),
                 vec![],
-                None,
             )?;
         }
         Ok(())
@@ -388,8 +377,9 @@ impl PrintHtmlOptions {
         //     name: 'I k2 logits_diff, true I',
         //     desc: 'I k2 logits_diff, true I [t: Cumulant] [s: ()]',
         //     color: 'red',
-        //     children: ['2', '3'],
+        //     children: [['2', ' ! input 1'], ['3']],
         //   }
+
         html.push_str("var datatoggle___js___ = {");
         let mut seen: HashSet<HashBytes> = HashSet::default();
         self.fill_overall_data(circuits.clone(), &mut html, &mut seen)?;
@@ -450,7 +440,7 @@ impl PrintHtmlOptions {
 
     pub fn repr_line(&self, circ: CircuitRc) -> Result<String> {
         let line_info = self.get_print_options().repr_line_info(circ.clone());
-        if let Some(name) = circ.info().name {
+        if let Some(name) = circ.name() {
             Ok(format!("'{name}'{}", &line_info?))
         } else {
             line_info
@@ -691,7 +681,7 @@ const JS: &str = r#"
   }
 
   // Parameters: id of the parent html element, id of the node to expand
-  function addChildtoggle___js___(parent_id, id) {
+  function addChildtoggle___js___(parent_id, id, child_info) {
     var child = document.createElement('li');
     child.classList.add('list');
 
@@ -709,7 +699,7 @@ const JS: &str = r#"
     var elementId = id + '@' + childData.displayedCount + '@' + seedtoggle___js___;
     child.id = elementId; // Assumes no "@" in ID
 
-    var desc = childData.desc;
+    var desc = childData.desc + child_info;
     if (childData.displayedCount > 0) {
       desc += ' (repeat)';
     }
@@ -763,8 +753,10 @@ const JS: &str = r#"
 
       let alreadExpanded = getLastChild(elt).children.length > 0;
       if (!alreadExpanded) {
-        datatoggle___js___[nodeId].children.forEach(function (childId) {
-          addChildtoggle___js___(elementId, childId);
+        datatoggle___js___[nodeId].children.forEach(function (childIdAndInfo) {
+          var childId = childIdAndInfo[0];
+          var childInfo = childIdAndInfo.slice(1).join(' ');
+          addChildtoggle___js___(elementId, childId, childInfo);
         });
       }
     }
@@ -792,7 +784,7 @@ const JS: &str = r#"
   
   rootstoggle___js___.forEach(function (root) {
     console.log(root);
-    addChildtoggle___js___("s"+seedtoggle___js___, root);
+    addChildtoggle___js___("s"+seedtoggle___js___, root, '');
   });
   toExpandtoggle___js___.forEach(function (expandElt) {
     console.log(expandElt);
