@@ -12,7 +12,7 @@ use rr_util::{
 };
 #[pyfunction]
 pub fn generalfunction_merge_inverses(circuit: &GeneralFunction) -> Option<CircuitRc> {
-    if circuit.nodes.len()==1 && circuit.spec.is_official() && let Circuit::GeneralFunction(inner)=&**circuit.nodes[0] && inner.spec.is_official() && OFFICIAL_GENERALFUNCTION_INVERSES.iter().any(|(a,b)|a==&circuit.spec.name()&&b==&inner.spec.name()) {
+    if circuit.nodes.len()==1 && circuit.spec.is_official() && let Circuit::GeneralFunction(inner)=&**circuit.nodes[0] && inner.spec.is_official() && OFFICIAL_GENERALFUNCTION_INVERSES.iter().any(|(a,b)|*a==&circuit.spec.name()&&*b==&inner.spec.name()) {
         return Some(inner.nodes[0].clone())
     }
     None
@@ -30,7 +30,7 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                     return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
-                        circuit.info().name,
+                        circuit.name_cloned(),
                     ));
                 }
             }
@@ -42,7 +42,7 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                     return Some(Scalar::nrc(
                         scalar,
                         circuit.info().shape.clone(),
-                        circuit.info().name,
+                        circuit.name_cloned(),
                     ));
                 }
             }
@@ -56,7 +56,7 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                             .iter()
                             .cloned()
                             .collect(),
-                        circuit.info().name,
+                        circuit.name_cloned(),
                     ));
                 }
             }
@@ -93,7 +93,7 @@ pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<Circuit
                     .extract(py)
                     .unwrap(),
                     sv![],
-                    node.info().name,
+                    node.name_cloned(),
                 )
                 .rc()
             })
