@@ -9,6 +9,7 @@ use threadpool::ThreadPool;
 use crate::{
     py_types::{Tensor, PY_UTILS},
     rrfs::get_rrfs_dir,
+    util::HashBytes,
 };
 
 pub fn get_tensor_cache_dir() -> String {
@@ -92,10 +93,10 @@ pub fn get_only_file_in_nested_dirs(dir: &str) -> Result<String> {
 }
 
 pub fn get_tensor_dir_tree(dir: &str, hash_prefix: &str) -> Result<Tensor> {
-    get_filename_dir_tree(dir, hash_prefix).and_then(|s| load_tensor(s))
+    get_filename_dir_tree(dir, hash_prefix).and_then(|(s, h)| load_tensor(s, Some(h)))
 }
 
-pub fn get_filename_dir_tree(dir: &str, hash_prefix: &str) -> Result<String> {
+pub fn get_filename_dir_tree(dir: &str, hash_prefix: &str) -> Result<(String, String)> {
     let (wholes, part) = split_prefix(&hash_prefix);
     let look_dir = dir.to_owned() + "/" + &wholes.join("/");
     let files =
@@ -107,23 +108,35 @@ pub fn get_filename_dir_tree(dir: &str, hash_prefix: &str) -> Result<String> {
         0 => {
             bail!(anyhow!("Found no tensors matching hash prefix"))
         }
-        1 => get_only_file_in_nested_dirs(
-            &(look_dir
+        1 => {
+            let longpath = look_dir
                 + "/"
                 + &matching_entries[0]
                     .as_ref()
                     .unwrap()
                     .file_name()
                     .to_str()
-                    .unwrap()),
-        ),
+                    .unwrap();
+            if matching_entries[0]
+                .as_ref()
+                .unwrap()
+                .file_type()
+                .unwrap()
+                .is_file()
+            {
+                Ok(longpath)
+            } else {
+                get_only_file_in_nested_dirs(&longpath)
+            }
+            .map(|file| (file.clone(), file[dir.len()..].replace("/", "")))
+        }
         _ => {
             bail!(anyhow!("Found multiple tensors matching hash prefix"))
         }
     }
 }
 
-pub fn load_tensor(dir: String) -> Result<Tensor> {
+pub fn load_tensor(dir: String, hash: Option<String>) -> Result<Tensor> {
     Python::with_gil(|py| {
         PY_UTILS
             .torch
@@ -133,6 +146,17 @@ pub fn load_tensor(dir: String) -> Result<Tensor> {
             .context(format!("Failed to load tensor from hash {}", &dir))?
             .extract(py)
             .context("Failed to extract pyobject in tensor from hash")
+            .and_then(|t: Tensor| {
+                let Some(hash) = hash else {
+                    return Ok(t);
+                };
+                let hash_vec: Vec<u8> = base16::decode(&hash).context("hash not base16")?;
+                let hash: HashBytes = HashBytes::try_from(hash_vec)
+                    .map_err(|e| anyhow!("wrong length hash {}", e.len()))?;
+                let mut t = t;
+                t.set_hash(Some(hash));
+                Ok(t)
+            })
     })
 }
 
@@ -172,10 +196,31 @@ fn copy_without_setting_permissions(local_filename: &str, remote_filename: &str)
     std::io::copy(&mut reader, &mut writer).context("writing contents failed")?;
     Ok(())
 }
+/// this is only useful bc querying multiple tensors in parallel is much faster than in series
+pub fn ensure_all_tensors_local(hashes: Vec<String>) -> Result<()> {
+    // println!("loading {} tensors", hashes.len());
+    let n_workers = 30;
+    let nitems = hashes.len();
+    let pool = ThreadPool::new(n_workers);
+    let (tx, rx) = channel();
+    for hash in hashes {
+        let tx = tx.clone();
+        pool.execute(move || {
+            let result = ensure_prefix_local(&hash).map(|_| ());
+            // println!("did");
+            tx.send(result).unwrap()
+        });
+    }
+    rx.iter().take(nitems).collect::<Result<()>>()
+}
 
 #[pyfunction(parallelism = "15")]
 pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
     let list_dir = get_tensor_cache_dir() + "/.unsynced_list";
+    if !fs::try_exists(&list_dir)? {
+        println!("didnt find any usynced list (~/tensors_by_hash_cache/.unsynced_list), returning");
+        return Ok(());
+    }
     println!("starting to read list");
     let all_unsynced = fs::read_to_string(&list_dir)?;
     println!("read list {}", all_unsynced.len());
@@ -194,7 +239,7 @@ pub fn sync_all_unsynced_tensors(parallelism: usize) -> Result<()> {
         let tx = tx.clone();
         pool.execute(move || {
             tx.send((move || {
-                let local_filename = get_filename_dir_tree(&get_tensor_cache_dir(), &line)?;
+                let local_filename = get_filename_dir_tree(&get_tensor_cache_dir(), &line)?.0;
                 let remote_filename =
                     local_filename.replace(&get_tensor_cache_dir(), &get_rrfs_tensor_db_dir());
                 let remote_filename_dir: Vec<_> = remote_filename.split("/").collect();
@@ -226,6 +271,26 @@ pub fn get_tensor_prefix(prefix: &str) -> Result<Tensor> {
     })
 }
 
+#[pyfunction]
+pub fn ensure_prefix_local(prefix: &str) -> Result<()> {
+    let cache_dir = get_tensor_cache_dir();
+    get_filename_dir_tree(&cache_dir, &prefix)
+        .map(|_| ())
+        .or_else(|_e| {
+            let remote_filename = get_filename_dir_tree(&get_rrfs_tensor_db_dir(), prefix)?.0;
+            let local_filename =
+                remote_filename.replace(&get_rrfs_tensor_db_dir(), &get_tensor_cache_dir());
+            let local_filename_dir: Vec<_> = local_filename.split("/").collect();
+            let local_filename_dir = local_filename_dir
+                [..local_filename_dir.len().checked_add_signed(-1).unwrap()]
+                .join("/");
+            fs::create_dir_all(&local_filename_dir).context("failed to create dir in tree")?;
+            copy_without_setting_permissions(&remote_filename, &local_filename)
+                .context("failed to copy")?;
+            Ok(())
+        })
+}
+
 #[pyfunction]
 pub fn migrate_tensors_from_old_dir(dir: &str) -> Result<()> {
     let files: Vec<String> = fs::read_dir(&dir)
@@ -240,7 +305,7 @@ pub fn migrate_tensors_from_old_dir(dir: &str) -> Result<()> {
     for file in files {
         if !lines.contains(&&file[..64]) {
             print!("O");
-            let tensor = load_tensor(dir.to_owned() + "/" + &file)?;
+            let tensor = load_tensor(dir.to_owned() + "/" + &file, None)?;
             save_tensor(tensor, false)?;
         } else {
             print!("_");
