@@ -244,7 +244,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(
-        circuit_rewrites::compiler_strip::strip_names_and_tags_py,
+        circuit_rewrites::compiler_strip::strip_names_py,
         m
     )?)?;
     m.add_function(wrap_pyfunction!(deep_maybe_distribute_py, m)?)?;
